"""
(C) @mrlokaman 
All rights reserved by LN Technical

"""

"""
MIT License

Copyright (c) 2021 Lntechnical

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""


# This Modules Helps To Get Youtube Video Data 


import re
import urllib.request
import json



class Search:
	
	def __init__(self,keywords:str , limit=15):
		
		search_keyword=keywords.replace(" ", "+")
		html = urllib.request.urlopen("https://www.youtube.com/results?search_query=" + search_keyword)
		self.limit= limit
		self.source = html.read().decode()
		
		
	def videos(self):
		"""
		[
  {
    "id": "P1uHDPpe_04",
    "title": "Build language translator with 3 lines of python code",
    "thumb": [
      "https://i.ytimg.com/vi/P1uHDPpe_04/hq720.jpg?sqp=-oaymwEjCOgCEMoBSFryq4qpAxUIARUAAAAAGAElAADIQj0AgKJDeAE=\\u0026rs=AOn4CLBffClhp28J0iQB2e8jtV_am6TO4g",
      "https://i.ytimg.com/vi/P1uHDPpe_04/hq720.jpg?sqp=-oaymwEXCNAFEJQDSFryq4qpAwkIARUAAIhCGAE=\\u0026rs=AOn4CLAld_3eDi_itf1WQJ9rF7TmXLCJmA"
    ],
    "simple_data": "Build language translator with 3 lines of python code by codebasics 8 months ago 4 minutes, 58 seconds 21,386 views"
  },
  {
    "id": "mwq5-01z1U0",
    "title": "Create a Simple Language Translator Bot using Bot Builder SDK and Google Translation API",
    "thumb": [
      "https://i.ytimg.com/vi/mwq5-01z1U0/hq720.jpg?sqp=-oaymwEjCOgCEMoBSFryq4qpAxUIARUAAAAAGAElAADIQj0AgKJDeAE=\\u0026rs=AOn4CLA6jeEFU7aMQqQNdwxoPf-IutXx8g",
      "https://i.ytimg.com/vi/mwq5-01z1U0/hq720.jpg?sqp=-oaymwEXCNAFEJQDSFryq4qpAwkIARUAAIhCGAE=\\u0026rs=AOn4CLDZoi0Vy4wS5peNhwfL2Idqkuptqg"
    ],
    "simple_data": "Create a Simple Language Translator Bot using Bot Builder SDK and Google Translation API by JD Bots 1 year ago 12 minutes, 55 seconds 594 views"
  },
  {
    "id": "PTAkiukJK7E",
    "title": "Creating a Telegram Bot in Python 3.9 Tutorial (Fast \\u0026 Easy)",
    "thumb": [
      "https://i.ytimg.com/vi/PTAkiukJK7E/hq720.jpg?sqp=-oaymwEjCOgCEMoBSFryq4qpAxUIARUAAAAAGAElAADIQj0AgKJDeAE=\\u0026rs=AOn4CLB9N4Z0WpfElA6UVVk2A7ye54ZveQ",
      "https://i.ytimg.com/vi/PTAkiukJK7E/hq720.jpg?sqp=-oaymwEXCNAFEJQDSFryq4qpAwkIARUAAIhCGAE=\\u0026rs=AOn4CLDOwR6QmwLlNIuvJSb4sRHQuH7L6Q"
    ],
    "simple_data": "Creating a Telegram Bot in Python 3.9 Tutorial (Fast \\u0026 Easy) by Code Palace 7 months ago 13 minutes, 7 seconds 64,519 views"
  },
  {
    "id": "I5iux1KXswk",
    "title": "how to translate any language on telegram app with translator bot on telegram - how to use telegram",
    "thumb": [
      "https://i.ytimg.com/vi/I5iux1KXswk/hq720.jpg?sqp=-oaymwEjCOgCEMoBSFryq4qpAxUIARUAAAAAGAElAADIQj0AgKJDeAE=\\u0026rs=AOn4CLAWge1QJjpIuvKcVRBIlxgMU-1jig",
      "https://i.ytimg.com/vi/I5iux1KXswk/hq720.jpg?sqp=-oaymwEXCNAFEJQDSFryq4qpAwkIARUAAIhCGAE=\\u0026rs=AOn4CLALlQl3kEP4oVN7JxnZH4D5JGualw"
    ],
    "simple_data": "how to translate any language on telegram app with translator bot on telegram - how to use telegram by TS Tech Talk 3 months ago 7 minutes, 24 seconds 7,771 views"
  },
  {
    "id": "CTwlDu6ik3s",
    "title": "Google Translate Telegram Bot | Nodejs \\u0026 Telegraf",
    "thumb": [
      "https://i.ytimg.com/vi/CTwlDu6ik3s/hq720.jpg?sqp=-oaymwEjCOgCEMoBSFryq4qpAxUIARUAAAAAGAElAADIQj0AgKJDeAE=\\u0026rs=AOn4CLDBuCSuLnUQfGwp5tA91JDcHrh3vg",
      "https://i.ytimg.com/vi/CTwlDu6ik3s/hq720.jpg?sqp=-oaymwEXCNAFEJQDSFryq4qpAwkIARUAAIhCGAE=\\u0026rs=AOn4CLC5Uu46b8qQ8-t-i9XSJyH11SF3Eg"
    ],
    "simple_data": "Google Translate Telegram Bot | Nodejs \\u0026 Telegraf by modos coding 1 week ago 3 minutes, 50 seconds 17 views"
  }
]
		
		"""		
		limit = self.limit
		source = self.source
		data  = re.findall('{\"videoRenderer\":{\"videoId\":\"(\S{11})\",\"thumbnail\":{\"thumbnails\":\[{\"url\":\"(\S+)\",\"width\":360,\"height\":202},{\"url\":\"(\S+)\",\"width\":720,\"height\":404}\]},\"title\":{\"runs\":\[{\"text\":\"(.+?)\"}\],\"accessibility\":{\"accessibilityData\":{\"label\":\"(.+?)\"}}},\"longBylineText\"',source)[:limit]
		data_ = []
		for i in data:
				js_data = {"id":"",
		            "title":"", 
		             "thumb" : "" ,
		              "simple_data":""}
				js_data['id'] = i[0]
				js_data['title'] = i[3]
				js_data['thumb'] = i[1],i[2]
				js_data['simple_data'] = i[4]
				data_.append(js_data)
		value =  json.dumps(data_ )
		return json.loads(value)
						


"""
(C) @mrlokaman 
All rights reserved by LN Technical

"""

"""
MIT License

Copyright (c) 2021 Lntechnical

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""


# This Modules Helps To Get Youtube Video Data 


import re
import urllib.request
import json

class Data:
      
      """
      	Get YouTube Video Data 
      	
      """
      def __init__(self,link):
      		headers = { 'User-Agent' : 'Mozilla/5.0 (Windows NT 6.1; Win64; x64)' }
      		res = urllib.request.Request(link, headers=headers)
      		html = urllib.request.urlopen(res)
      		self.source = html.read().decode()
      		     		
      		
      # Get Video id 
      def id(self):
      	try:
      		videodetails=re.findall("\"videoDetails\":\{(.+?),\"isOwnerViewing", self.source)[0]
      	except:
      		return None
      	try:
      		#Get id , title From videodetails variable
      		id = re.findall("\"videoId\":\"(\S{11})",videodetails)[0]
      		return id 
      	except:
      		return None
      		
      	
      			
      # Get Video Title 	
      def title(self):
      	try:
      		videodetails=re.findall("\"videoDetails\":\{(.+?),\"isOwnerViewing", self.source)[0]
      	except:
      		return None
      	try:
      		#Get id , title From videodetails variable
      		title = re.findall("\"title\":\"(.+?)\",",videodetails)[0]
      		return title 
      	except:
      		return None
    
      
      
      #Get Thumbnails Link From Youtube Video 
      def thumb(self):
      		try :
      			thumb= re.findall("\"thumbnails\":\[\{\"url\":\"(.+?)\",\"width",self.source )[0]
      			return thumb
      		except:
      			return None
      		
      		
     # Get Video Publish Date 	
      def publish_date(self):
      	try:
      		publish_date = re.findall("\"publishDate\":\"(\d{4}-\d{2}-\d{2})", self.source)[0]
      		return publish_date
      	except:
      		return None
      
      def tag(self):
      	tag =re.findall("itag\":251,\"url\":\"(.+?)\",\"mimeType\"",self.source)
      	return tag
      	
      
      # Get Views Of the Video
      def views(self):
      	try:      		 
      		views = re.findall("\"viewCount\":\"(\d+)",self.source)[0]
      		return views
      	except:
      		return None
      
      
      # Get Category Of The Video 
      def category(self):
      	try:      		
      		category = re.findall("\"category\":\"(.+?)\",", self.source)[0]
      		return category
      	except:
      		return None
      
      
      
      # Get Channel Name 
      def channel_name(self):
      		try:
      			channelName = e.findall("\"channelName\":\"(.+?)\",", self.source)[0]
      			return channelName
      		except:
      			try:
      				channelName = re.findall("\"ownerChannelName\":\"(.+?)\",\"uploadDate",self.source)[0]
      				return channelName
      			except:
      				return None
     
      				 				
      				
      #Get likes Of The Video 				
      def likes(self):
      		try:
      			likes = re.findall("accessibilityData\":{\"label\":\"(\S+) likes", self.source)[0]
      			return likes
      		except:
      			return None
      		
     
      		 		
      #Get dislikes Of The Video    		
      def dislikes(self): 
      	try:
      		dislikes = re.findall("accessibilityData\":{\"label\":\"(\S+) dislike", self.source)[0]
      		return dislikes
      	except:
      		return None
      		
      		
      #Get Subscriber of The video 		
      def subscriber(self):
      		try:
      			sub = re.findall("subscriberCountText\":{\"accessibility\":{\"accessibilityData\":{\"label\":\"(.+?)\"}}", self.source)[0]  
      			return sub
      		except:
      			try:
      				sub = re.findall("accessibilityData\":{\"label\":\"(\S+) subscribers", self.source)[0]   				
      				return sub
      			except:
      				return None 
      			
      def data(self):
      	try:
      		# Get Video Details 
      		videodetails=re.findall("\"videoDetails\":\{(.+?),\"isOwnerViewing", self.source)[0]
      	except:
      		videodetails = None
      	try:
      		#Get id , title From videodetails variable
      		id = re.findall("\"videoId\":\"(\S{11})",videodetails)[0]
      		title = re.findall("\"title\":\"(.+?)\",",videodetails)[0]
      	except:
      		title = None
      		id = None
      	try :
      		#Get Thumbnails Link From Youtube Video 
      		thumb= re.findall("\"thumbnails\":\[\{\"url\":\"(.+?)\",\"width",self.source )[0]
      	except:
      		thumb = None
      	try:
      		# Get Video Publish Date 
      		publish_date = re.findall("\"publishDate\":\"(\d{4}-\d{2}-\d{2})", self.source)[0]
      	except:
      		publish_date = None
      	try:
      		# Get Views Of the Video 
      		views = re.findall("\"viewCount\":\"(\d+)",self.source)[0]
      	except:
      		views = None
      	try:
      		# Get Category Of The Video 
      		category = re.findall("\"category\":\"(.+?)\",", self.source)[0]
      	except:
      		category = None       		
      	try:
      		# Get Channel Name 
      		channelName = e.findall("\"channelName\":\"(.+?)\",", self.source)[0]
      	except:
      		try:
      			channelName = re.findall("\"ownerChannelName\":\"(.+?)\",\"uploadDate",self.source)[0]
      		except:
      			channelName = None
      			
      	try:
      		#Get likes Of The Video 
      		likes = re.findall("accessibilityData\":{\"label\":\"(\S+) likes", self.source)[0]
      	except:
      		likes = None
      	try:
      		#Get dislikes Of The Video 
      		dislikes = re.findall("accessibilityData\":{\"label\":\"(\S+) dislike", self.source)[0]
      	except:
      		dislikes = None 
      	try:
      		#Get Subscriber of The video 
      		sub = re.findall("accessibilityData\":{\"label\":\"(\S+) subscribers", self.source)[0]
      	except:
      		try:
      			sub = re.findall("subscriberCountText\":{\"accessibility\":{\"accessibilityData\":{\"label\":\"(.+?)\"}}", self.source)[0]  
      		except:
      			sub = None
      		
      	DATA = { 
      	             "id": id,
      	             "title": title,
      	             "thumbnails": thumb,
      	             "views": views,
      	             "likes": likes,
      	             "dislikes":dislikes,
      	             "publishdate": publish_date,
      	             "category": category,
      	             "channel_name": channelName,
      	             "subscriber":sub
      	        }
      	 
      	return DATA